/*
 *  GDMdensities.cpp
 *
 *  Copyright (C) 2008, Simon H. Rasmussen,  MSc. Student at The Bioinformatics Centre,
 *  University of Copenhagen.
 *
 *  This file is part of Mocapy++.
 *
 *  Mocapy++ is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Mocapy++ is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Mocapy++.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <sstream>
#include <iostream>
#include "GDMdensities.h"

namespace mocapy
{

GDMdensities::GDMdensities()
{
    initialize();
}

  //GDMdensities::~GDMdensities(){
  //  for(uint i=0; i<node_size; i++)
  //    {
  //      densities[i].cleanup();
  //    }    
  //}

  void GDMdensities::initialize()
  {
    type = GDM;
    output_size = dim;
  }

  
  GDMdensities::GDMdensities(uint dim,uint sample_size)
  {
    this->dim = dim;
    //    this->node_size = node_size;
    //cpd.set_shape(node_size, 2 * (dim - 1));
    this->sample_size = sample_size;
    initialize();
}

void GDMdensities::set_parameters(MDArray<double> & pars)
{
    // Check cpd is right shape
    vector<uint> shape = pars.get_shape();
    assert(shape[1] == 2 * (dim - 1));
    //    printf("Node size: %d \n",node_size);
    for(uint i = 0;i < node_size;i++){
      cpd.push_back(pars.get_view(i));
      // make new MultinomialDensity objects
    //    _create_densities();
    }
    if (!cpd.empty()){
      assert(!cpd.empty());
      //     printf("cpd is NOT empty \n");
    }
    else{
      // printf("cpd is empty \n");
    }
}

void GDMdensities::construct(vector<uint> & parent_sizes)
{
    // GDMNode always has a single parent
    assert(parent_sizes.size() == 1);
    node_size = parent_sizes[0];
    // vector of GDMDensity objects
    densities.resize(node_size);
    for(uint i = 0;i < parent_sizes.size();i++){
    }
    _create_densities();
}

void GDMdensities::_create_densities()
{
  first = true;
  ulong seed = 4450;
  MDArray<double> pars;
  vector<double> alphas;
  vector<double> betas;
  // Initialize the density objects and fill the density vector
  for(uint i=0; i<node_size; i++)
    {
      densities[i] = GDMdensity(dim, seed, sample_size);
      init.push_back(true);
    }
}

void GDMdensities::estimate(vector<MDArray<double> > & ess)
{
  assert(!ess.empty());
  int n = ess.size();

  if(first){
    first = false;
    int p;
    // vector<int> indices;  
    //  for(uint i = 0;i < densities.size();i++){
    //  indices.push_back(0);
    //}
    MDArray<double> tmp;

    for(int i = 0; i < n; i++){
      tmp = ess[i];
      p = tmp[0];
      vector<double> cvec;
      //      cout << indices[p] << " ";
      for(uint j = 0; j < dim; j++){
	cvec.push_back(tmp[j + 1]);
	//	cout << cvec[j];
      }
      // cout << endl;
      densities[p].add_countv(cvec);
      //      indices[p]++;
    }
  }
  //  assert(false);
  int stocEM_samples = 500;
  // init = true;
  // densities[p].stoc_EM(stocEM_samples, init[p]);
  // init[p] = false;
  //for(uint i = 0;i < 99;i++){

    //  densities[p].stoc_EM(stocEM_samples,init[p]);
    //}
  for(int p = 0;p < node_size;p++){
    /*
 if(!init[p]){
      for(int i=0;i<5;i++){
	cout << cpd[p][i] << " ";
      }
      cout << endl;
    }
    */    //    cout << " p " << p << " Init[p] " << init[p] << endl;
    vector< vector<double> > estpars = densities[p].stoc_EM(stocEM_samples,init[p]);
    vector< vector<double> > max_estpars = estpars;
    double maxlik = -99999999;
    double lik = densities[p].sum_loglik();
    if (lik > maxlik){
      max_estpars = estpars; 
    }
    init[p] = false;  
    for (int i = 0;i<0;i++){
      estpars = densities[p].stoc_EM(stocEM_samples,init[p]);
      lik = densities[p].sum_loglik();
      //      if (lik > maxlik){
      //	max_estpars = estpars;
      // }
    }
    // estpars = max_estpars;
    densities[p].set_parameters(estpars[0],estpars[1]);
    MDArray<double> pars;
    pars.set_shape(2 * (dim - 1));

    for(uint i = 0; i < dim - 1; i++){
      pars[i] = estpars[0][i];
      pars[i + dim - 1] = estpars[1][i];
    }
    if (cpd.size() < node_size){
      cpd.push_back(pars);
    }
    else{
      cpd[p] = pars;
    }
  }
}

vector<double> GDMdensities::sample(vector<double> & pv)
{
  assert(!cpd.empty());
  uint p=(unsigned int) (pv[0]+0.1);
  int sumOfCounts = 14;

  MDArray<double> pars = cpd[p];

  vector<double> alphas;
  vector<double> betas;
  
  for(uint j = 0; j < dim-1; j++)
    {
      alphas.push_back(pars[j]);
      betas.push_back(pars[j + (dim - 1)]);
    }
  vector<double> sample;
  sample = densities[p].Sample_mult(alphas,betas,sumOfCounts);
  /* for(int i =0;i<5;i++){
  printf(" %f",sample[i]);
  }
  printf("\n");
  */
  return sample;
}

double GDMdensities::get_lik(vector<double> & ptv, bool log_space)
{

    uint p=(unsigned int) (ptv[0]+0.1);
    // Start at 1 (last argument): first value is parent
    double ll = densities[p].loglik(ptv);
    if (log_space)
        return ll;
    else
        return exp(ll);
}

vector<MDArray<double> > GDMdensities::get_parameters()
{
  //    vector<MDArray<double> > v(1);
  //    v[0] = cpd;
  return cpd;
}

ostream& operator<<(ostream& output, const GDMdensities& a)  {
    output << "Node: Generalized Dirichlet Multinomial, size: " << a.node_size << " " << a.dim << endl;

    if (a.cpd.empty()){
      output << "Uninitialized." << endl;
    }
    else{
      for(uint i = 0;i < a.cpd.size();i++){
        output << a.cpd[i].tostring();
      }
    }
    return output;
}

}
