/*
 *  GDMdensities.cpp
 *
 *  Copyright (C) 2008, Simon H. Rasmussen, MSc. Student at The Bioinformatics Centre,
 *  University of Copenhagen.
 *
 *  This file is part of Mocapy++.
 *
 *  Mocapy++ is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Mocapy++ is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Mocapy++.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <boost/math/special_functions/gamma.hpp>
#include <vector>
#include <iostream>
#include <sstream>
#include <fstream>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "GDMdensity.h"
//#include "../utils/mdarray.h"
#include "../utils/utils.h"
//#include <tr1/random>
#include <boost/random/gamma_distribution.hpp>
#include <boost/random/variate_generator.hpp>
#include <boost/random/mersenne_twister.hpp>
//#include <random>
#include <stdio.h>
#include <stdlib.h>
//#include <gsl/gsl_rng.h>
//#include <gsl/gsl_randist.h>
//#include <gsl/gsl_sf_gamma.h>
#include <math.h>
#include <algorithm>
#include <utility>

namespace mocapy
{

  using namespace std;
  
  GDMdensity::GDMdensity(){
    dim=2;
    init_pars();
  }
  
  GDMdensity::GDMdensity(uint dim, ulong s,int nn){
    this->dim = dim;
    n = nn;
    //    const gsl_rng_type * T = gsl_rng_taus;
    //  const gsl_rng_type * T = gsl_rng_ranlxd2;
    ulong seed = s;
    //    r = gsl_rng_alloc (T);
    //   gsl_rng_set (r, seed);
    //  cvector = new double*[n];
    // pvector = new double*[n]; 
    //    rMeans = new double[dim-1];
    //    rVars = new double[dim-1];
    cache_initialized = false;
    init_pars();
    rng1.seed(356);
    rng2.seed(2387);
  }
  
  void printv(vector<double> & v1, vector<double> & v2){
    int d = v1.size();
    cout << "[";
    for(int i = 0;i < d;i++){
      printf("%.2f ",v1[i]);
    }
    cout << ",\n ";
    for(int i = 0;i < d;i++){
      printf("%.2f ",v2[i]);
    }
    cout << "]" << endl;
  }
  /*
  double GDMdensity::sample_beta(double a,double b){
    double rn = gsl_ran_beta(r,a,b);
    if (a<0 || b<0){
      cout << "in sample_beta: rn alphas betas are " << rn << " " << a << " " << b << endl;
      return -1; 
    }
    return rn;
  }
  */
  double GDMdensity::sample_beta2(double a,double b){
    //tr1::mt19937 eng1;
    //tr1::mt19937 eng2;
    //tr1::gamma_distribution<double> X(a);
    // tr1::gamma_distribution<double> Y(b);
    // double rn = X(eng1)/(X(eng1)+Y(eng2));
    boost::gamma_distribution<double> gam1(a);     
    boost::variate_generator<boost::mt19937&, boost::gamma_distribution<double> > gamma1(rng1, gam1);
    boost::gamma_distribution<double> gam2(b);     
    boost::variate_generator<boost::mt19937&, boost::gamma_distribution<double> > gamma2(rng2, gam2);
    double X = gamma1();
    double Y = gamma2();
    //    cout << "X " << X << " Y " << Y << endl;
    double rn = X/(X+Y);
    //    cout << "X " << X << " Y " << Y << " alpha " << a << " beta " << b << endl;
    if (a<0 || b<0){
      cout << "in sample_beta: rn alphas betas are " << rn << " " << a << " " << b << endl;
      return -1; 
    }
    return rn;
  }

  vector<double> GDMdensity::Sample_Gendir(vector<double> & as,vector<double> & bs){
    // Calculates a Generalized Dirichlet sample sampling from the beta distribution.
    double sum;
    vector<double> theta;
    for (int i = 0;i<dim;i++){
      theta.push_back(0);
    }
    /*    if(get_rand() > .95){
    cout << "Alphas ";
    for (int i = 0;i<dim-1;i++){
      cout << as[i] << " ";
    }
      cout << endl;
    cout << "Betas ";
    for (int i = 0;i<dim-1;i++){
      cout << bs[i] << " ";
    }
      cout << endl;
    }
    */
    theta[0] = sample_beta2(as[0], bs[0]);
    sum = theta[0];
    
    for(int i = 1;i < dim-1;i++){
      theta[i] = sample_beta2(as[i], bs[i]) * (1-sum);
      sum = sum + theta[i];
    }
    
    sum = 0;
    for (int i = 0;i<dim-1;i++){
      sum = sum + theta[i];
    }
    theta[dim-1] = (1-sum);
    return theta;
  }
  
  void GDMdensity::scanData(){
    // sets up cache for lnGammai()
    if (!cache_initialized){
      cache_initialized = true;
      int  max = 0;
      int sum;
      for(int i = 0;i<n;i++){
	sum = 0;
	for(int j = 0;j<dim;j++){
	  sum = sum + (int) (cvector[i][j] + .1);
	}
	if (sum > max){
	  max = sum;
	}
      }
      Nmax = max;
      for(int k = 0;k < Nmax-1;k++){
	lnGammaCache.push_back(-1);
      }
    }
  }

  int GDMdensity::place_in_sequence(vector<double> & a, double item){
    // A very simple, but linear implementation of bisect.
    int i = 0;
    while(a[i] <= item && (i < dim-1)){
      i++;
    }
    return i;
  }

  vector<double> GDMdensity::Sample_mult(vector<double> & as, vector<double> & bs,int N){
    // A method for sampling multinomial vectors.
    /*  printf("GMD_SM før set_pars\n");
    set_parameters(as,bs);
    vector<double> theta = Sample_Gendir(as, bs);
    vector<double> cumsum;
    double sum = theta[0];
      int j = 1;
    double rnum;
    vector<double> multsample;
    
    for(int i = 0;i < dim;i++){ 
      while(j <= i){
	sum = sum + theta[j];
	j = j + 1;
      }
      cumsum.push_back(sum);
      multsample.push_back(0);
    }
    
    int choice;
    for(int i = 0;i<N;i++){
      // random number between 0 and 1.
      rnum = gsl_rng_uniform (r);
      choice = place_in_sequence(cumsum,rnum);
      multsample[choice] = multsample[choice] + 1; 
    }
    printf("size %d \n",multsample.size());
    */
    CPD cumsum;
    double rnum = .1;
    vector<uint> shape;
    shape.push_back(dim);
    cumsum.set_shape(shape);
    vector<double> theta = Sample_Gendir(as, bs);
    for(int i = 0;i<dim;i++){
      cumsum[i] =  theta[i];
    }
    cumsum.cumsum(); 
    vector<double> multsample;
    for(int i = 0;i < dim;i++){ 
      multsample.push_back(1.0);
    }
    int choice;
    for(int i = 0;i<N;i++){
      //      random number between 0 and 1.
      //      rnum = gsl_rng_uniform (r);
      rnum = get_rand();
      choice = cumsum.bisect(rnum);
      multsample[choice] = multsample[choice] + 1; 
    }
    return multsample;
  }
/*
// In case we want to estimate GD parameters directly.
void GDMdensity::add_probvector(int i,double *p){
  pvector[i] = p;
}
*/
  void GDMdensity::add_countv(vector<double> & c){
    // Add vector of counts.
    cvector.push_back(c);
    pvector.push_back(c);
    /*      for(int j = 0;j<dim;j++){
      cout << c[j];
    } 
          cout << endl;
    */
    n = cvector.size();
    //   cout << " i " << i << endl;
    
  }

vector< vector<double> > GDMdensity::reject( vector< vector<double> > & p){
  // In case estimates are nan of <= 0 we reject them in the current iteration
  // and use the ones from previous iteration.
  double init = 1.02;
  double lim = 0.1;
  for(int d = 0;d < dim-1;d++){
    if (p[0][d] < lim || isnan(p[0][d])){
      p[0][d] = init;
    }
    if (p[1][d] < lim || isnan(p[1][d])){
      p[1][d] = init;
    }
  }
  return p;
}
  /*
double GDMdensity::lngammaf(double c){
  // Accessing the log(gamma(x))-function in gsl for floats and thereby no caching.
  if (c <= 2.0){
    return 0;
  }
  return boost::math::lgamma(c);
}
  */
double GDMdensity::lngammai(int c){
  // Accessing the log(gamma(x))-function in gsl for integers, uses caching.
  if (c <= 2){
    return 0;
  }
  if (lnGammaCache[c-3] == -1){
    lnGammaCache[c-3] = boost::math::lgamma(c);
  }
  return lnGammaCache[c-3];
}

double GDMdensity::getMult_loglik(vector<double> & countv ,vector<double> & f){
  // Calculates the multinomial log-likelihood.
  double ll;
  int c;
  int s = 0;

  for(int i = 0;i < dim;i++){
    s = s + (int) (countv[i]+.1);
  }
  ll = lngammai(s+1);

  for(int i = 0;i < dim;i++){
    c = (int) (countv[i]+.1);
    ll = ll - lngammai(c+1);;
    ll = ll + c *  log(f[i]);
  }
  return ll;
}

double GDMdensity::loglik(vector<double> & countv){
  // Calculates the GDM log-likelihood
  int s = 0;
  int c;
  double ll = 0;
  double t2 = 0;
  double t1 = 0;
  double ap = 0;
  double bp = 0;
  for(int i = 0;i < dim;i++){
    s = s + (int) countv[i]+.1;
  }
  if(cache_initialized){
    ll = lngammai(s+1);
  }
  else{
    ll = lgamma(s+1);
  }
  for(int i = 0;i < dim;i++){
    c = (int) countv[i]+.1;
    if(cache_initialized){
      ll = ll - lngammai(c+1);
    }
    else{
      ll = ll - lgamma(c+1);
    } 
    
    if (i < dim - 1){
      t1 = t1 + boost::math::lgamma(alphas[i] + betas[i]) - boost::math::lgamma(alphas[i]) - boost::math::lgamma(betas[i]);
      ap = alphas[i] + c;
      s = s - c;
      bp = betas[i] + s;
      t2 = t2 - boost::math::lgamma(ap+bp) + boost::math::lgamma(ap) + boost::math::lgamma(bp);
    }
  } 
  ll = ll + t1 + t2;
  return ll;
}

double GDMdensity::sum_loglik(){
  double sum=0;
  vector<double> vec;
  for(int i = 0;i < n;i++){
    sum = sum + loglik(cvector[i]);
  }
  //  cout << " sum_loglik sum " << sum << endl;
  return sum;
}

double GDMdensity::covsum(int d1,double *means,vector< vector<double> > & vec){
  // Calculates number of dimensions that each have negative covariance with.
  double cov;
  double dl1;
  double dl2;
  double cnum;

  cnum = 0;
  for (int d2 = 0;d2 < dim; d2++){
    cov = 0;
    for (int i = 0;i < n;i++){
      dl1 = (vec[i][d1] - means[d1]);
      dl2 = (vec[i][d2] - means[d2]);
      cov = cov + dl1*dl2;
      cov=cov/(double) (n - 1);
    }
    if (d2 != d1){
      if (cov < 0){
	cnum = cnum - 1;
      }
    }
  }
  return cnum; 
}

bool GDMdensity::sortdim(vector< vector<double> > & vec){
  // Sorts dimensions with regard to count of negative covariance with other dimensions.
  vector< pair<double, int> > covs; 
  double *means = new double[dim]; 
  for(int i = 0;i < dim;i++){
    means[i] = mean(i,vec);
  }
  for (int d1 = 0;d1 < dim; d1++){
    covs.push_back(pair<double, int>(covsum(d1,means,vec), d1));
  }
  stable_sort(covs.begin(),covs.end());

  vector<double> covvec;
  vector<pair<double, int> >::const_iterator itr;

  for(itr = covs.begin(); itr != covs.end(); ++itr){
    covvec.push_back((*itr).first);
    idxvec.push_back((*itr).second);
  }
  delete[] means;

  for(int i = 0;i < dim ;i++){
    if (idxvec[i] != i){
      return true;
    }
  }
  return false;
}
		
void GDMdensity::swap_dims(vector< int > & idx,double **vec){
  // Swapps the dimensions of the dynamically allocated array given as argument
  // according to the order defined by the idx vector.
  double *tmp;
  double *tmp1;
  for (int i = 0;i < n; i++){
    tmp1 = new double[dim];
    tmp = vec[i];
    for (int j = 0;j < dim; j++){
      tmp1[j] =  tmp[idx[j]];
    }
    delete[] vec[i];
    vec[i] = tmp1;
  }
}

void GDMdensity::set_parameters(vector<double> & as,vector<double> & bs){
  // Makes it posible to directly set parameters in a density.
  // Check cpd is right shape and not yet set.
  if ((int) as.size() == dim - 1 && (int) bs.size() == dim - 1 && alphas.empty() && betas.empty()){
    alphas=as;
    betas=bs;
  }
}

void GDMdensity::init_pars(){
  // Initializes the parameters to Dirichlet parameters (not Generalized Dirichlet).
  int a = 6;
  if (alphas.size() == 0 && betas.size() == 0){
    for(int i = 0;i<dim-1;i++){
      alphas.push_back(a);
      betas.push_back(a*dim - a*i);
    }
  }
  else{
    for(int i = 0;i<dim-1;i++){
      alphas[i] = a;
      betas[i] = a*dim - a*i;
    }
  }
}

vector< vector<double> > GDMdensity::stoc_EM(int samples, bool init){
  // Estimation of multinomial parameter set used for estimating GD parameters.
  // This is implemnted using Metropolis-Hastings sampling.
  double p1;
  double p2;
  double ran;
  int num = 0;

  
  /*  for(int i = 0;i < n;i++){
    cout << " Cvector " << i << " ";
    for(int k = 0;k < dim;k++){
      cout << cvector[i][k];
    }
    cout << endl;
  }
  */
  //  for(int k = 0;k < dim;k++){
  //    cout << cvector[0][k];
  //  }
  //  cout << endl;
  
  vector< vector<double> > p;
  vector< vector<double> > pars;
  vector< vector<double> > tmppars;
  if (init){
    scanData();
    //    if (sortdim(cvector)){
    //     cout << "Dimension has been sorted, dimensions will be swapped" << endl;
    //pintvec(idxvec);
    //     swap_dims(idxvec,cvector);
    // }
  }
     
  if (init){
    init_pars();
    pars.push_back(alphas);
    pars.push_back(betas);
  }

  for(int i = 0;i < n;i++){
    vector <double> f1 = Sample_Gendir(alphas, betas);
    p1 = getMult_loglik(cvector[i], f1);
    for(int j = 0;j < samples;j++){
      for(int j = 0;j<dim-1;j++){
	// Report if nan's appear in parameters. 
	//If this happens a lot the user needs to rethink model.
	if (isnan(alphas[i]) || isnan(betas[i])){
	  cout << "in Stoc: alphas betas i "<< alphas[i] << " " << betas[i] << " " << i << endl;
	}
      }
      // Sample multinomial parameter vector.
      vector<double> f2 = Sample_Gendir(alphas, betas);
      p2 = getMult_loglik(cvector[i], f2);

      // if log-likelihood is better for f2, it is chosen.
      if (p2 > p1){
	f1 = f2;
	p1 = p2;
      }
      else{
	// if not we do a random choice.
	//	ran=gsl_rng_uniform (r);
	ran=get_rand();

	double rr = log((long double) ran);
	num++;
	if (rr < p2 - p1){
	  f1 = f2;
	  p1 = p2;
	}
      }
    }
    pvector[i] = f1;
    //  printf("pvector[i]: ");
    //  for (int j = 0;j < dim;j++){
    // printf("%f ",pvector[i][j]);
    // }
    //printf("%d \n",n);
    //    ll = ll + loglik(cvector[i]);
  }
  pars = estimate_pars(init);
  pars = reject(pars);
  alphas = pars[0];
  betas = pars[1];
  return pars;
}

double GDMdensity::mean(int d,vector< vector<double> > & vec){
  // Calculates mean of a vector.
  double mean = 0;
  for(int i = 0;i < n;i++){
    mean = mean + vec[i][d];
  }
  mean = mean/n;
  return mean;
}

double GDMdensity::var(double mean,int d,vector< vector<double> > & pvec){
  // Calculates variance of a vector, given the mean.
  double var=0;
  double delta;

  for (int i = 0;i<n;i++){
    delta = (pvec[i][d] - mean);
    var = var + delta*delta;
  }
  var=var/(n-1);

  return var; 
}

vector< vector<double> > GDMdensity::estimate_pars(bool init){
  /* OLD CODE THAT I'M NOT YET COMPLETELY SURE NOT IS NEEDED.
     if (init){
    cout << "Initialize" << endl;
    for(int i = 0;i<dim-1;i++){
      alphas[i] = 1;
      betas[i] = 1;
    }
  }
  */
  // Estimate GD parameters 
  vector< vector<double> > dpars;
  dpars.push_back(alphas);
  dpars.push_back(betas);
  double sumvar = 0;
  double t = 1;
  double u = 1;
  double e;
  double e2;
  double v;
  double den;
  vector< vector<double> > pars;

  for (int i = 0;i<dim-1;i++){
    e = mean(i,pvector);
    v = var(e,i,pvector);
    sumvar=sumvar + v;
    e2 = e*e;
    den = v*t+e2*t-e2*u;
    // If nan's appear here estimation has crashed.
    if (isnan(e) || isnan(v) || isnan(e2)){
      cout << "NAN e var e2: " << e << " " << v << " " << e2 << endl; 
    }   
    alphas[i] = (e*(-v+e*u-e2)/den);
    betas[i] = ((t-e)*(-v+e*u-e2)/den);
    // If nan's appear here estimation has crashed.
    if (isnan(alphas[i]) || isnan(betas[i])){
      cout << "in Estimate_pars: alphas betas i "<< alphas[i] << " " << betas[i] << " " << i << endl;
    }
    t = t * betas[i]/(alphas[i]+betas[i]);
    u = u * (betas[i]+1)/(alphas[i]+betas[i]+1);
  }
  
  pars.push_back(alphas);
  pars.push_back(betas);
  
  return pars;
}
  /*
  void GDMdensity::cleanup(){
    // Deletes pointers and pointer arrays to avoid leaks.
    for (int i = 0;i < n;i++){
      delete[] pvector[i]; 
      delete[] cvector[i];
    }
    delete[] pvector; 
    delete[] cvector;
  }
  */
}
