#ifndef GDMDENSITY_H_
#define GDMDENSITY_H_

#include <vector>
#include <iostream>
#include <sstream>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include "../utils/utils.h"
#include <boost/random/mersenne_twister.hpp>
//#include "../utils/mdarray.h"
namespace mocapy
{
  using namespace std;
  class GDMdensity{

  public:
    GDMdensity();
    GDMdensity(uint dim, ulong s, int nn);
    void add_countv(vector<double> & c);
    void set_parameters(vector<double> & as,vector<double> & bs);
    void init_pars();
    vector<double> Sample_mult(vector<double> & alphas,vector<double> & betas, int N);
    double getMult_loglik(vector<double> & countv, vector<double> & f);
    vector< vector<double> > stoc_EM(int samples,bool init);
    //    void cleanup();
    double loglik(vector<double> & countv);
    double sum_loglik();

  private:
    double sample_beta2(double a,double b);
    vector<double> Sample_Gendir(vector<double> & alphas, vector<double> & betas);    
    vector< vector<double> > reject(vector< vector<double> > & p);
    vector< vector<double> > estimate_pars(bool init);
    double sample_beta(double a,double b);
    void scanData();
    double lngammaf(double c);
    double lngammai(int c);
    double mean(int dim, vector< vector<double> > & vec);
    double var(double mean, int dim,vector< vector<double> > & pvec);
    int place_in_sequence(vector<double> & a, double item);
    bool sortdim(vector< vector<double> > & vec);
    double covsum(int d1,double *means,vector< vector<double> > & vec);
    void swap_dims(vector< int > & idx,double **vec);


    boost::mt19937 rng1;                 
    boost::mt19937 rng2;                 
    vector<int> idxvec;
    vector< vector<double> > pvector;
    vector< vector<double> > cvector;
    //    double *rMeans;
    //    double *rVars;
    vector<double> alphas;
    vector<double> betas;
    int n;
    int Nmax;
    vector<double> lnGammaCache;
    int dim;
    bool cache_initialized;

    // Persistence
    friend class boost::serialization::access;
    template<class Archive>
      void serialize(Archive & ar, const unsigned int version);
    
  };

  template<class Archive>
    void GDMdensity::serialize(Archive & ar, const unsigned int version)
    {
      ar & idxvec;
      ar & pvector;
      ar & cvector;
      ar & alphas;
      ar & betas;
      ar & n;
      ar & Nmax;
      ar & lnGammaCache;
      ar & dim;
      ar & rng1;
      ar & rng2;
      ar & cache_initialized;
    }
}
#endif /* GDMDENSITY_H_ */
