/*
 * bippodensities.h
 *
 *  Copyright (C) 2009, Thomas Hamelryck, The Bioinformatics Centre, 
 *  University of Copenhagen.
 *
 *  This file is part of Mocapy++.
 *
 *  Mocapy++ is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Mocapy++ is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Mocapy++.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef BIPPODENSITIES_H_
#define BIPPODENSITIES_H_

#include <boost/archive/text_oarchive.hpp>
#include <boost/archive/text_iarchive.hpp>
#include <fstream>

#include "../framework/densitiesbase.h"

#include "bippo.h"
#include "bippoess.h"

namespace mocapy {

class BippoDensities;
std::ostream& operator<<(std::ostream&, const BippoDensities&);

class BippoDensities : public DensitiesBase {
public:
	BippoDensities(vector<double> new_user_lambdas = vector<double> (),
        vector<double> new_user_thetas = vector<double> (),
        vector<double> new_user_ns = vector<double> ());
	virtual ~BippoDensities() {};

	// Initializes the Density arrays
	void construct(vector<uint> & parent_sizes);

	// Parameter estimation based on the ESS
	void estimate(vector<MDArray<double> > & ess);

	// Return a sample, based on indicated parent values
	vector<double> sample(vector<double> & pv);

	// Return likelihood, that is: P(child|parents)
	double get_lik(vector<double> & ptv, bool log=false);

	// Return the distribution's parameters
	vector< MDArray<double> > get_parameters();

	friend ostream& operator<< (ostream& output, const BippoDensities& a);

	// Persistence
	friend class boost::serialization::access;
    template<class Archive>
    void serialize(Archive & ar, const unsigned int version);

private:
    // Bippo parameters
    vector<double> user_lambdas;
    vector<double> lambdas;
    vector<double> user_thetas;
    vector<double> thetas;
    vector<double> user_ns;
    vector<double> ns;

    // List of densities: one for each parent state
    vector<Bippo> densities;

    void initialize();
};

template<class Archive>
void BippoDensities::serialize(Archive & ar, const unsigned int version) {
    ar & boost::serialization::base_object<DensitiesBase>(*this);
    ar & user_lambdas;
    ar & lambdas;
    ar & user_thetas;
    ar & thetas;
    ar & user_ns;
    ar & ns;
    ar & densities;
}

}

#endif /* BIPPODENSITIES_H_ */
